from matrixprofile.algorithms.top_k_discords import top_k_discords as discords
from matrixprofile.algorithms.top_k_motifs import top_k_motifs as motifs
from matrixprofile.algorithms.snippets import snippets
from matrixprofile.algorithms.regimes import extract_regimes as regimes
from matrixprofile.algorithms.statistics import statistics
from matrixprofile.algorithms.hierarchical_clustering import (
    hierarchical_clusters
)