# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto_messages.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto_messages.proto',
  package='mpf',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x14proto_messages.proto\x12\x03mpf\"<\n\x0c\x44oubleMatrix\x12\x0c\n\x04rows\x18\x01 \x01(\x04\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x04\x12\x10\n\x04\x64\x61ta\x18\x03 \x03(\x01\x42\x02\x10\x01\"9\n\tIntMatrix\x12\x0c\n\x04rows\x18\x01 \x01(\x04\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x04\x12\x10\n\x04\x64\x61ta\x18\x03 \x03(\x03\x42\x02\x10\x01\"$\n\x08Location\x12\x0b\n\x03row\x18\x01 \x01(\x04\x12\x0b\n\x03\x63ol\x18\x02 \x01(\x04\"H\n\x05Motif\x12\x1d\n\x06motifs\x18\x01 \x03(\x0b\x32\r.mpf.Location\x12 \n\tneighbors\x18\x02 \x03(\x0b\x32\r.mpf.Location\"\xde\x01\n\rMatrixProfile\x12\x1d\n\x02mp\x18\x01 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1a\n\x02pi\x18\x02 \x01(\x0b\x32\x0e.mpf.IntMatrix\x12\x1e\n\x03lmp\x18\x03 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1b\n\x03lpi\x18\x04 \x01(\x0b\x32\x0e.mpf.IntMatrix\x12\x1e\n\x03rmp\x18\x05 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1b\n\x03rpi\x18\x06 \x01(\x0b\x32\x0e.mpf.IntMatrix\x12\n\n\x02\x65z\x18\t \x01(\x04\x12\x0c\n\x04join\x18\n \x01(\x08\"P\n\x10PanMatrixProfile\x12\x1e\n\x03pmp\x18\x01 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1c\n\x04pmpi\x18\x02 \x01(\x0b\x32\x0e.mpf.IntMatrix\"\xf8\x02\n\tMPFOutput\x12\r\n\x05klass\x18\x01 \x01(\t\x12\x11\n\talgorithm\x18\x02 \x01(\t\x12\x0e\n\x06metric\x18\x03 \x01(\t\x12\x12\n\nsample_pct\x18\x04 \x01(\x01\x12\x13\n\x07windows\x18\x05 \x03(\x04\x42\x02\x10\x01\x12\x1d\n\x02ts\x18\x06 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12 \n\x05query\x18\x07 \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1e\n\x02mp\x18\x08 \x01(\x0b\x32\x12.mpf.MatrixProfile\x12\"\n\x03pmp\x18\t \x01(\x0b\x32\x15.mpf.PanMatrixProfile\x12\x1a\n\x06motifs\x18\n \x03(\x0b\x32\n.mpf.Motif\x12\x1f\n\x08\x64iscords\x18\x0b \x03(\x0b\x32\r.mpf.Location\x12\x1e\n\x03\x63mp\x18\x0c \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x1d\n\x02\x61v\x18\r \x01(\x0b\x32\x11.mpf.DoubleMatrix\x12\x0f\n\x07\x61v_type\x18\x0e \x01(\tb\x06proto3'
)




_DOUBLEMATRIX = _descriptor.Descriptor(
  name='DoubleMatrix',
  full_name='mpf.DoubleMatrix',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rows', full_name='mpf.DoubleMatrix.rows', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cols', full_name='mpf.DoubleMatrix.cols', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='mpf.DoubleMatrix.data', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=89,
)


_INTMATRIX = _descriptor.Descriptor(
  name='IntMatrix',
  full_name='mpf.IntMatrix',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rows', full_name='mpf.IntMatrix.rows', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cols', full_name='mpf.IntMatrix.cols', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='mpf.IntMatrix.data', index=2,
      number=3, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=148,
)


_LOCATION = _descriptor.Descriptor(
  name='Location',
  full_name='mpf.Location',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='row', full_name='mpf.Location.row', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='col', full_name='mpf.Location.col', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=150,
  serialized_end=186,
)


_MOTIF = _descriptor.Descriptor(
  name='Motif',
  full_name='mpf.Motif',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='motifs', full_name='mpf.Motif.motifs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='neighbors', full_name='mpf.Motif.neighbors', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=188,
  serialized_end=260,
)


_MATRIXPROFILE = _descriptor.Descriptor(
  name='MatrixProfile',
  full_name='mpf.MatrixProfile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mp', full_name='mpf.MatrixProfile.mp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pi', full_name='mpf.MatrixProfile.pi', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lmp', full_name='mpf.MatrixProfile.lmp', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lpi', full_name='mpf.MatrixProfile.lpi', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rmp', full_name='mpf.MatrixProfile.rmp', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rpi', full_name='mpf.MatrixProfile.rpi', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ez', full_name='mpf.MatrixProfile.ez', index=6,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='join', full_name='mpf.MatrixProfile.join', index=7,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=263,
  serialized_end=485,
)


_PANMATRIXPROFILE = _descriptor.Descriptor(
  name='PanMatrixProfile',
  full_name='mpf.PanMatrixProfile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pmp', full_name='mpf.PanMatrixProfile.pmp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pmpi', full_name='mpf.PanMatrixProfile.pmpi', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=487,
  serialized_end=567,
)


_MPFOUTPUT = _descriptor.Descriptor(
  name='MPFOutput',
  full_name='mpf.MPFOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='klass', full_name='mpf.MPFOutput.klass', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='algorithm', full_name='mpf.MPFOutput.algorithm', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metric', full_name='mpf.MPFOutput.metric', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample_pct', full_name='mpf.MPFOutput.sample_pct', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='windows', full_name='mpf.MPFOutput.windows', index=4,
      number=5, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ts', full_name='mpf.MPFOutput.ts', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query', full_name='mpf.MPFOutput.query', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mp', full_name='mpf.MPFOutput.mp', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pmp', full_name='mpf.MPFOutput.pmp', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='motifs', full_name='mpf.MPFOutput.motifs', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='discords', full_name='mpf.MPFOutput.discords', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cmp', full_name='mpf.MPFOutput.cmp', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='av', full_name='mpf.MPFOutput.av', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='av_type', full_name='mpf.MPFOutput.av_type', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=570,
  serialized_end=946,
)

_MOTIF.fields_by_name['motifs'].message_type = _LOCATION
_MOTIF.fields_by_name['neighbors'].message_type = _LOCATION
_MATRIXPROFILE.fields_by_name['mp'].message_type = _DOUBLEMATRIX
_MATRIXPROFILE.fields_by_name['pi'].message_type = _INTMATRIX
_MATRIXPROFILE.fields_by_name['lmp'].message_type = _DOUBLEMATRIX
_MATRIXPROFILE.fields_by_name['lpi'].message_type = _INTMATRIX
_MATRIXPROFILE.fields_by_name['rmp'].message_type = _DOUBLEMATRIX
_MATRIXPROFILE.fields_by_name['rpi'].message_type = _INTMATRIX
_PANMATRIXPROFILE.fields_by_name['pmp'].message_type = _DOUBLEMATRIX
_PANMATRIXPROFILE.fields_by_name['pmpi'].message_type = _INTMATRIX
_MPFOUTPUT.fields_by_name['ts'].message_type = _DOUBLEMATRIX
_MPFOUTPUT.fields_by_name['query'].message_type = _DOUBLEMATRIX
_MPFOUTPUT.fields_by_name['mp'].message_type = _MATRIXPROFILE
_MPFOUTPUT.fields_by_name['pmp'].message_type = _PANMATRIXPROFILE
_MPFOUTPUT.fields_by_name['motifs'].message_type = _MOTIF
_MPFOUTPUT.fields_by_name['discords'].message_type = _LOCATION
_MPFOUTPUT.fields_by_name['cmp'].message_type = _DOUBLEMATRIX
_MPFOUTPUT.fields_by_name['av'].message_type = _DOUBLEMATRIX
DESCRIPTOR.message_types_by_name['DoubleMatrix'] = _DOUBLEMATRIX
DESCRIPTOR.message_types_by_name['IntMatrix'] = _INTMATRIX
DESCRIPTOR.message_types_by_name['Location'] = _LOCATION
DESCRIPTOR.message_types_by_name['Motif'] = _MOTIF
DESCRIPTOR.message_types_by_name['MatrixProfile'] = _MATRIXPROFILE
DESCRIPTOR.message_types_by_name['PanMatrixProfile'] = _PANMATRIXPROFILE
DESCRIPTOR.message_types_by_name['MPFOutput'] = _MPFOUTPUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DoubleMatrix = _reflection.GeneratedProtocolMessageType('DoubleMatrix', (_message.Message,), {
  'DESCRIPTOR' : _DOUBLEMATRIX,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.DoubleMatrix)
  })
_sym_db.RegisterMessage(DoubleMatrix)

IntMatrix = _reflection.GeneratedProtocolMessageType('IntMatrix', (_message.Message,), {
  'DESCRIPTOR' : _INTMATRIX,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.IntMatrix)
  })
_sym_db.RegisterMessage(IntMatrix)

Location = _reflection.GeneratedProtocolMessageType('Location', (_message.Message,), {
  'DESCRIPTOR' : _LOCATION,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.Location)
  })
_sym_db.RegisterMessage(Location)

Motif = _reflection.GeneratedProtocolMessageType('Motif', (_message.Message,), {
  'DESCRIPTOR' : _MOTIF,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.Motif)
  })
_sym_db.RegisterMessage(Motif)

MatrixProfile = _reflection.GeneratedProtocolMessageType('MatrixProfile', (_message.Message,), {
  'DESCRIPTOR' : _MATRIXPROFILE,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.MatrixProfile)
  })
_sym_db.RegisterMessage(MatrixProfile)

PanMatrixProfile = _reflection.GeneratedProtocolMessageType('PanMatrixProfile', (_message.Message,), {
  'DESCRIPTOR' : _PANMATRIXPROFILE,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.PanMatrixProfile)
  })
_sym_db.RegisterMessage(PanMatrixProfile)

MPFOutput = _reflection.GeneratedProtocolMessageType('MPFOutput', (_message.Message,), {
  'DESCRIPTOR' : _MPFOUTPUT,
  '__module__' : 'proto_messages_pb2'
  # @@protoc_insertion_point(class_scope:mpf.MPFOutput)
  })
_sym_db.RegisterMessage(MPFOutput)


_DOUBLEMATRIX.fields_by_name['data']._options = None
_INTMATRIX.fields_by_name['data']._options = None
_MPFOUTPUT.fields_by_name['windows']._options = None
# @@protoc_insertion_point(module_scope)
